#include <vector>
using namespace std;

#define USE_CONSOLE
#include <allegro.h>

#include <stdio.h>

struct Pix 
{ 
    int x; 
    int y; 
    int col; 
};


int main(int argc, char** argv)
{
    //    install_allegro(SYSTEM_NONE, &errno, atexit);
    allegro_init();

	install_keyboard();
    set_color_depth(32);
	set_gfx_mode(GFX_SAFE, 640, 480, 0, 0);

    clear_to_color(screen, makecol(255,255,255));

    if(argc != 4) {
        printf("Usage: %s <infile> <cname> <outfile>\n", argv[0]);
        printf("Reads the image in <infile> and outputs a cupid format datastructure in <outfile> with the given <cname>\n");
    }

    char* infilename = argv[1];
    char* cname = argv[2];
    char* outfilename = argv[3];

    BITMAP* bmp = load_bitmap(infilename, NULL);

    if(!bmp) {
        printf("Error reading %s!\n", infilename);
        return -1;
    }


    FILE* f = fopen(outfilename, "w");

    if(!f) {
        printf("Error opening %s for writing\n", outfilename);
    }

    printf("Generating function make_bitmap_%s to %s\n", cname, outfilename);

    fprintf(f,"#include <allegro.h>\n\n");
    fprintf(f,"#define L(x1,y,x2,r,g,b) hline(bmp, x1, y, x2, makecol(r,g,b))\n");
    fprintf(f,"#define P(x,y,r,g,b) putpixel(bmp, x, y, makecol(r,g,b))\n");

    fprintf(f,"// Painstakingly hand generated by Damyan from %s\n\n", infilename);
    fprintf(f,"BITMAP* make_bitmap_%s()\n", cname);
    fprintf(f,"{\n");
    fprintf(f,"    BITMAP* bmp = create_bitmap(%d, %d);\n", bmp->w, bmp->h);
    fprintf(f,"    \n");
    fprintf(f,"    // Make background transparent\n");
    fprintf(f,"    clear_to_color(bmp, 0);\n\n");
    fprintf(f,"    // All the horizontal blocks of the same color\n");


    blit(bmp, screen, 0, 0, bmp->w, 0, bmp->w, bmp->h);


    vector<Pix> pixs;

    int x,y;

    for(y=0; y<bmp->h; y++) {
        x = 0;

        int current_col;
        int count;
        int start_x;

        while(x < bmp->w) {
            count=1;
            start_x = x;

            current_col = getpixel(bmp, x, y);
            x++;

            while(getpixel(bmp,x,y) == current_col) {
                x++;
                count++;
            }

            int r,g,b,a;
            r = getr(current_col);
            g = getg(current_col);
            b = getb(current_col);
            a = geta(current_col);

            bool isalphad = false;

            if(bitmap_color_depth(bmp) == 32)
                isalphad = (a < 128);
            else
                isalphad = ! ((r == 255) && (g == 0) && (b == 255));

            if(!isalphad) {
                if(count > 1) {
                    
                    //                    fprintf(f,"    hline(bmp, %d, %d, %d, makecol(%d,%d,%d));\n",
                    fprintf(f,"    L(%d,%d,%d,%d,%d,%d);\n",
                            start_x, y, start_x+count-1, r,g,b);
                    
                    hline(screen, start_x, y, start_x+count-1, current_col);
                }
                else {
                    Pix p;
                    p.x = start_x;
                    p.y = y;
                    p.col = current_col;
                    pixs.push_back(p);
                }
            }

        }
    }


    if(pixs.size() > 0) 
        fprintf(f,"\n    // Now all the single pixels that got missed out\n");

    vector<Pix>::iterator it = pixs.begin();
    while(it != pixs.end()) {
        putpixel(screen, it->x, it->y, it->col);

        int r,g,b;
        r = getr(it->col);
        g = getg(it->col);
        b = getb(it->col);
        
        //        fprintf(f,"    putpixel(bmp, %d, %d, makecol(%d,%d,%d));\n", 
        fprintf(f,"    P(%d,%d,%d,%d,%d);\n",
                it->x, it->y, r, g, b);
        it++;
    }

    fprintf(f,"\n\n    // phew...never again\n");
    fprintf(f,"    return bmp;\n");
    fprintf(f,"}\n\n");

    readkey();
    return 0;
}

END_OF_MAIN();

