// info about levels

#ifndef __LEVEL_H__
#define __LEVEL_H__


#include "main.h"
#include "objects.h"



// all info about a level: name, walls, enemy positions, and so on
struct Level
{
	const char *mName;			// human-readable name

	const char *mSourceName;	// source filename
	
	int mNumEdges;				// number of edges

	int *mPoints;				// array of x/y start/finish pairs of pairs (quadrupleds?)

	int mNumObjects;			// number of objects of each type

	ObjData *mObjData;			// array of object data

    int mHighscore;
    char mHighscoreName[10];
};



// list of all the levels
extern Level *level_table[];



// pointer to the current level
extern Level *level;



// scrolling position
extern Vector scroll;
extern Vector scroll_vel;



// init and draw functions
void init_level();
void update_level();
void render_level(BITMAP *bmp);


// collision detection returns true if there is a collisionIt is up to
// the caller to deal with extraction etc.
struct LevelCollision
{
    float dist;
    Vector normal;

    LevelCollision(float d, Vector& n)
        : dist(d), normal(n)
    {
    }
};

bool check_level_collision(class Thing* thing, vector<LevelCollision>& collisions);
bool check_level_collision_line(const Vector &p1, const Vector &p2, vector<LevelCollision>& collisions, float tolerance = 0);

void keep_points_inside_world(Vector* oldpoints, Vector* points, int numpoints);



#endif		// __LEVEL_H__


