// generates the plasma texture (I think this is called Perlin noise but could be wrong)

#include "plasma.h"



BITMAP *plasma = NULL;

BITMAP *make_bitmap_rivet();



static BITMAP *create_noise(int size)
{
	BITMAP *bmp = create_bitmap(size, size);

	for (int x=0; x<size; x++) {
		for (int y=0; y<size; y++) {
			bmp->line[y][x] = rand() & 63;
		}
	}
	
	return bmp;
}



static int bilinear_lookup(BITMAP *bmp, float x, float y)
{
	int ix = (int)x;
	int iy = (int)y;
	
	float fx = x - ix;
	float fy = y - iy;
	
	float c1a = bmp->line[iy][ix];
	float c1b = bmp->line[iy][(ix+1) % bmp->w];

	float c2a = bmp->line[(iy+1) % bmp->h][ix];
	float c2b = bmp->line[(iy+1) % bmp->h][(ix+1) % bmp->w];

	float c1 = c1a*(1-fx) + c1b*fx;
	float c2 = c2a*(1-fx) + c2b*fx;

	return (int)(c1*(1-fy) + c2*fy);
}



static BITMAP *smooth_noise(int size)
{
	BITMAP *noise = create_noise(size);
	
	BITMAP *bmp = create_bitmap(256, 256);
	
	for (int x=0; x<256; x++) {
		for (int y=0; y<256; y++) {
			bmp->line[y][x] = bilinear_lookup(noise, (float)(x*size)/256.0, (float)(y*size)/256.0);
		}
	}
	
	destroy_bitmap(noise);
	
	return bmp;
}



void init_plasma()
{
	BITMAP *b[6];

	static int weights[ARRAY_SIZE(b)] =
	{
		1,
		1,
		2,
		8,
		6,
		0
	};

	int i;

	for (i=0; i<ARRAY_SIZE(b); i++)
		b[i] = smooth_noise(256 >> i);

	plasma = create_bitmap(256, 256);
	
	for (int x=0; x<256; x++) {
		for (int y=0; y<256; y++) {
			int c = 0;
			int total = 0;

			for (i=0; i<ARRAY_SIZE(b); i++) {
				c += b[i]->line[y][x] * weights[i];
				total += weights[i];
			}
			
			c /= total;

			plasma->line[y][x] = MID(0, c, 63);
		}
	}
	
	for (i=0; i<ARRAY_SIZE(b); i++)
		destroy_bitmap(b[i]);

	BITMAP *rivet = make_bitmap_rivet();
	
	draw_sprite(plasma, rivet, 0,   0);
	draw_sprite(plasma, rivet, 128, 0);
	draw_sprite(plasma, rivet, 0,   128);
	draw_sprite(plasma, rivet, 128, 128);

	draw_sprite(plasma, rivet, 64,  64);
	draw_sprite(plasma, rivet, 192, 64);
	draw_sprite(plasma, rivet, 64,  192);
	draw_sprite(plasma, rivet, 192, 192);
	
	destroy_bitmap(rivet);
}
